@@ -1,8 +1,7 @@
-import sys, re, copy
 import numpy as np
+from copy import deepcopy
 
 import cctk
-from cctk import Molecule
 from cctk.helper_functions import align_matrices
 
 
@@ -50,7 +49,7 @@ class Ensemble:
             new = type(self)(name=self.name) # will return either Ensemble or subclass thereof
             new.add_molecule(mol, properties=prop)
             return new
-        elif isinstance(key, Molecule):
+        elif isinstance(key, cctk.Molecule):
             idx = self.molecule_list().index(key)
             return self[idx]
         elif isinstance(key, (list, np.ndarray)):
@@ -122,11 +121,22 @@ class Ensemble:
         """
         Returns ``True`` if property is defined for index ``idx`` and ``False`` otherwise.
         """
-        if prop in self[idx].properties_list()[0]:
+        combined = self.combined_properties()
+        if prop in combined:
             return True
         else:
             return False
 
+    def combined_properties(self):
+        """
+        Returns a dictionary containing the most up-to-date version of each property.
+        """
+        combined = dict()
+        for p in self.properties_list():
+            combined = {**combined, **p}
+
+        return combined
+
     def get_property(self, idx, prop):
         """
         """
@@ -219,8 +229,8 @@ class Ensemble:
             else:
                 raise ValueError(f"cannot index with type {str(type(key))}")
 
-        def __setitem__(self, key):
-            raise ValueError("cannot set molecule properties this way; use ensemble.set_property_dict(molecule, property_dict) instead")
+        def __setitem__(self, key, item):
+            raise LookupError("cannot set molecule properties this way; use ensemble.set_property_dict(molecule, property_dict) instead")
 
         def _check_key(self, key, n_items):
             assert -n_items <= key < n_items, f"key {key} is out of range...must be between {-n_items} and {n_items-1} inclusive"
@@ -273,11 +283,11 @@ class Ensemble:
             properties (dict): property name (str) to property value
             copy (bool): whether to store an independent copy of the molecule
         """
-        if not isinstance(molecule, Molecule):
+        if not isinstance(molecule, cctk.Molecule):
             raise TypeError("molecule is not a Molecule - so it can't be added!")
 
         if copy:
-            molecule = copy.deepcopy(molecule)
+            molecule = deepcopy(molecule)
 
         if properties is None:
             #### empty dicts all point to the same memory address by default, so need to prevent that behavior by initializing non-empty dict
@@ -404,7 +414,7 @@ class ConformationalEnsemble(Ensemble):
         optionally computing the root-mean-square distance between each
         geometry and the reference geometry.
 
-	Alignments are based on `atom_numbers`.
+        Alignments are based on `atom_numbers`.
         The current ensemble will not be altered.  RMSDs will be calculated over the
         comparison atoms only.
 
@@ -436,7 +446,7 @@ class ConformationalEnsemble(Ensemble):
         assert len(comparison_atoms) >= 3, f"need at least 3 atoms for alignment, but only got {len(comparison_atoms)}"
 
         # duplicate the ensemble
-        new_ensemble = copy.deepcopy(self)
+        new_ensemble = deepcopy(self)
 
         # translate all molecules to the origin
         # with respect to the comparison atoms
@@ -461,7 +471,8 @@ class ConformationalEnsemble(Ensemble):
             new_geometry = align_matrices(partial_geometry, full_geometry, partial_template_geometry)
             molecule.geometry = new_geometry
             if compute_RMSD:
-                after_RMSD = cctk.helper_functions.compute_RMSD(new_ensemble.molecules[to_geometry], new_ensemble.molecules[i], comparison_atoms)
+                partial_geometry = new_geometry[comparison_atoms]
+                after_RMSD = cctk.helper_functions.compute_RMSD(partial_template_geometry, partial_geometry)
                 after_RMSDs.append(after_RMSD)
             assert len(molecule.geometry) == n_atoms, f"wrong number of geometry elements! expected {n_atoms}, got {len(molecule.geometry)}"
 
@@ -518,19 +529,32 @@ class ConformationalEnsemble(Ensemble):
             energies = old_ensemble[:,"energy"]
             sorted_indices = list(np.argsort(energies))
 
+        # boolean indexing noticeably faster
+        idxs = np.array(comparison_atoms)
+        mask = np.zeros(old_ensemble.molecules[0].geometry.shape[0], dtype=bool)
+        mask[idxs - 1] = True
+
+        partial_geoms = [m.geometry[mask] for m in old_ensemble.molecules]
+        new_partial_geoms = []
+
         # add molecules one by one
         new_ensemble = ConformationalEnsemble()
         for i in sorted_indices:
-            candidate_molecule = old_ensemble.molecules[i]
-            candidate_molecule_properties = old_ensemble.get_properties_dict(candidate_molecule)
             ok_to_add = True
-            for existing_molecule in new_ensemble.molecules:
-                candidate_rmsd = cctk.helper_functions.compute_RMSD(candidate_molecule, existing_molecule, comparison_atoms, checks=False)
+
+            for existing_molecule in new_partial_geoms:
+                candidate_rmsd = cctk.helper_functions.compute_RMSD(partial_geoms[i], existing_molecule, checks=False)
                 if candidate_rmsd < RMSD_cutoff:
                     ok_to_add = False
                     break
+
             if ok_to_add:
+                candidate_molecule = old_ensemble.molecules[i]
+                candidate_molecule_properties = old_ensemble.get_properties_dict(candidate_molecule)
+
                 new_ensemble.add_molecule(candidate_molecule, candidate_molecule_properties)
+                new_partial_geoms.append(candidate_molecule.geometry[mask])
+
         return new_ensemble
 
     def get_geometric_parameters(self, parameter, atom1, atom2, atom3=None, atom4=None):
@@ -547,21 +571,20 @@ class ConformationalEnsemble(Ensemble):
         Returns:
             a list of the specified parameter's values for each geometry
         """
-        output = [None] * len(self.molecules)
-        for index, molecule in enumerate(self.molecules):
+        output = [None] * len(self)
+        for index, molecule in enumerate(self.molecule_list()):
             if parameter == "distance":
                 output[index] = molecule.get_distance(atom1, atom2)
             elif parameter == "angle":
-                if atom3 == None:
+                if atom3 is None:
                     raise ValueError("need atom3 to calculate angle!")
                 output[index] = molecule.get_angle(atom1, atom2, atom3)
             elif parameter == "dihedral":
-                if (atom3 == None) or (atom4 == None):
+                if (atom3 is None) or (atom4 is None):
                     raise ValueError("need atom3 and atom4 to calculate dihedral!")
                 output[index] = molecule.get_dihedral(atom1, atom2, atom3, atom4)
             else:
-                ValueError("Invalid parameter {}!".format(parameter))
+                raise ValueError(f"Invalid parameter {parameter}!")
 
         return output
 
-
