@@ -1,4 +1,4 @@
-import math, copy, scipy
+import math, copy, re
 import numpy as np
 import networkx as nx
 from scipy.spatial.distance import cdist
@@ -13,9 +13,11 @@ from cctk.helper_functions import (
     compute_dihedral_between,
     compute_unit_vector,
     get_covalent_radius,
+    get_vdw_radius,
     get_isotopic_distribution,
     compute_chirality,
 )
+import cctk.topology as top
 
 class Molecule:
     """
@@ -34,6 +36,7 @@ class Molecule:
         charge (int): the charge of the molecule
         multiplicity (int): the spin state of the molecule (1 corresponds to singlet, 2 to doublet, 3 to triplet, etc. -- so a multiplicity of 1 is equivalent to S=0)
         checks (bool): whether to check that the constructor parameters are valid
+        vibrational_modes (list of cctk.VibrationalMode): vibrational modes
     """
 
     def __init__(self, atomic_numbers, geometry, name=None, bonds=None, charge=0, multiplicity=1, checks=True):
@@ -79,6 +82,8 @@ class Molecule:
         self.multiplicity = multiplicity
         self.charge = charge
 
+        self.vibrational_modes = list()
+
         if isinstance(bonds, nx.Graph):
             self.bonds = bonds
         elif isinstance(bonds, (list,np.ndarray,nx.classes.reportviews.EdgeView)):
@@ -110,7 +115,8 @@ class Molecule:
 
     def assign_connectivity(self, cutoff=0.2, periodic_boundary_conditions=None):
         """
-        Automatically recalculates bonds based on covalent radii. If two atoms are closer than the sum of their covalent radii + ``cutoff`` Angstroms, then they are considered bonded.
+        Automatically recalculates bonds based on covalent radii. If two atoms are closer than the sum of their covalent radii + ``cutoff`` Angstroms,
+        then they are considered bonded.
 
         Args:
             cutoff (float): the threshold (in Angstroms) for how close two covalent radii must be to be considered bonded
@@ -375,7 +381,7 @@ class Molecule:
             atom1 = atoms[0]
             atom2 = atoms[1]
 
-        assert isinstance(distance, (float, int)), "need distance to set distance"
+        assert isinstance(distance, (float, int, np.number)), "need distance to set distance"
 
         self._check_atom_number(atom1)
         self._check_atom_number(atom2)
@@ -450,7 +456,7 @@ class Molecule:
             atom2 = atoms[1]
             atom3 = atoms[2]
 
-        assert isinstance(angle, (float, int)), "need angle to set angle"
+        assert isinstance(angle, (float, int, np.number)), "need angle to set angle"
 
         self._check_atom_number(atom1)
         self._check_atom_number(atom2)
@@ -557,7 +563,7 @@ class Molecule:
             atom3 = atoms[2]
             atom4 = atoms[3]
 
-        assert isinstance(dihedral, (float, int)), "need angle to set dihedral angle"
+        assert isinstance(dihedral, (float, int, np.number)), "need angle to set dihedral angle"
 
         # check atom numbers
         self._check_atom_number(atom1)
@@ -1252,12 +1258,52 @@ class Molecule:
 
         return Molecule(atoms, geoms, charge=charge, multiplicity=multiplicity)
 
-    def volume(self):
+    def volume(self, pts_per_angstrom=10, qhull=False):
         """
-        Returns volume calculated from ``scipy.spatial.ConvexHull``. Not a perfect approximation (assumes all atoms are points).
+        Returns volume calculated using the Gavezotti algorithm (JACS, 1983, 105, 5220). Relatively slow.
+        If MemoryError, defaults to a qhull-based approach (accurate in the limit as number of atoms goes to infinity)
+
+        Args:
+            pts_per_angstrom (int): how many grid points to use per Å - time scales as O(n**3) so be careful!
+            qhull (bool): use faster QHull algorithm
+
+        Returns:
+            volume in Å**3
         """
-        hull = scipy.spatial.ConvexHull(self.geometry.view(np.ndarray))
-        return hull.volume
+        if not qhull:
+            try:
+                assert isinstance(pts_per_angstrom, int), "Need an integer number of pts per Å!"
+                assert pts_per_angstrom > 0, "Need a positive integer of pts per Å!"
+
+                box_max = np.max(self.geometry.view(np.ndarray), axis=0) + 4
+                box_min = np.min(self.geometry.view(np.ndarray), axis=0) - 4
+
+                box_volume = (box_max[0] - box_min[0]) * (box_max[1] - box_min[1]) * (box_max[2] - box_min[2])
+
+                x_vals = np.linspace(box_min[0], box_max[0], int((box_max[0] - box_min[0]) * pts_per_angstrom))
+                y_vals = np.linspace(box_min[1], box_max[1], int((box_max[1] - box_min[1]) * pts_per_angstrom))
+                z_vals = np.linspace(box_min[2], box_max[2], int((box_max[2] - box_min[2]) * pts_per_angstrom))
+
+                # h4ck3r
+                box_pts = np.stack([np.ravel(a) for a in np.meshgrid(x_vals, y_vals, z_vals)], axis=-1)
+
+                # caching to speed call
+                vdw_radii = {z: get_vdw_radius(z) for z in set(self.atomic_numbers)}
+                radii_per_atom = np.array([vdw_radii[z] for z in self.atomic_numbers]).reshape(-1,1)
+
+                # this is the slow part since it's approximately a zillion operations
+                dists_per_atom = cdist(self.geometry.view(np.ndarray), box_pts)
+                occupied = np.sum(np.max(dists_per_atom < radii_per_atom, axis=0))
+
+                percent_occupied = occupied / box_pts.shape[0]
+                return percent_occupied * box_volume
+            except MemoryError:
+                qhull = True
+
+        if qhull:
+            import scipy
+            hull = scipy.spatial.ConvexHull(self.geometry.view(np.ndarray))
+            return hull.volume
 
     def swap_atom_numbers(self, atom1, atom2):
         """
@@ -1325,7 +1371,7 @@ class Molecule:
         mol = cctk.Group.add_group_to_molecule(mol, group1, mmap3[h2])
 
         #### relabel new graph to match original molecule
-        which = self._get_stereogenic_centers()
+        which = top.get_stereogenic_centers(self)
         which.remove(center_atom)
         return mol.renumber_to_match(self, check_chirality=which)
 
@@ -1336,7 +1382,7 @@ class Molecule:
         Args:
             model (cctk.Molecule): isomorphic molecule to renumber by
             check_chirality (list of atomic numbers): atomic numbers to check, to prevent inversion due to graph isomorphism.
-                Alternatively ``None`` will prevent any checking and "all" will use ``self._get_exchangable_centers()``.
+                Alternatively ``None`` will prevent any checking and "all" will use ``cctk.topology.get_exchangable_centers()``.
 
         Returns:
             new ``Molecule`` object
@@ -1362,29 +1408,29 @@ class Molecule:
         mol.bonds = nx.relabel_nodes(self.bonds, mapping=inv_mapping, copy=True)
 
         if check_chirality == "all":
-            check_chirality = mol._get_exchangeable_centers()
+            check_chirality = top.get_exchangeable_centers(mol)
 
         #### diastereotopic protons get scrambled by the above code so we gotta go through and fix all of them
         #### this happens because networkx doesn't store chirality - a known limitation of graph molecular encoding!
         if isinstance(check_chirality, list):
             #### find all the differences and exchange them
-            model_report = model.get_chirality_report(check_chirality)
+            model_report = top.get_chirality_report(model, check_chirality)
 
             #### generate all meso ring permutations
-            candidates = mol.flip_meso_rings(atoms=check_chirality)
+            candidates = top.flip_meso_rings(mol, atoms=check_chirality)
 
             #### for each, try flipping configuration of all centers
             for candidate in candidates:
-                report = candidate.get_chirality_report(check_chirality)
+                report = top.get_chirality_report(candidate, check_chirality)
                 for center in check_chirality:
                     if model_report[center] != report[center]:
                         try:
-                            candidate = candidate.exchange_identical_substituents(center)
+                            candidate = top.exchange_identical_substituents(candidate, center)
                         except ValueError as e:
                             break
 
                 #### check that we actually fixed all the problems
-                mol_report = candidate.get_chirality_report(check_chirality)
+                mol_report = top.get_chirality_report(candidate, check_chirality)
                 all_good = True
                 for center in check_chirality:
                     if mol_report[center] != model_report[center]:
@@ -1396,108 +1442,6 @@ class Molecule:
 
         raise ValueError("can't get a proper renumbering: are you *sure* these two molecules can have the same chirality?")
 
-    def _get_stereogenic_centers(self):
-        """
-        Returns every atom making 4 or more bonds. A bit misleading, since diastereotopic protons/meso protons are also counted.
-        """
-        assert self.bonds.number_of_edges() > 0, "need a bond graph to perform this operation -- try calling self.assign_connectivity()!"
-        num_neighbors = np.array([len(list(self.bonds[x])) for x in range(1, self.num_atoms() + 1)])
-        return [int(x) for x in list(np.ravel(np.argwhere(num_neighbors >= 4)) + 1)] # love me some off-by-one indexing errors
-
-    def _get_exchangeable_centers(self):
-        """
-        Returns all atoms making 4 or more bonds that have two isomorphic substituents, i.e. where renumbering could be broken.
-        """
-        centers = self._get_stereogenic_centers()
-        exchangeable_centers = []
-        for center in centers:
-            try:
-                mol = self.exchange_identical_substituents(center)
-                exchangeable_centers.append(center)
-                continue
-            except Exception as e:
-                pass
-
-            mols = self.flip_meso_rings(atoms=[center])
-            if len(mols) > 1:
-                exchangeable_centers.append(center)
-
-        return exchangeable_centers
-
-    def get_chirality_report(self, centers=None):
-        """
-        Computes chirality at stereogenic centers.
-
-        Args:
-            centers (list): atomic numbers to check. defaults to all centers with 4+ substituents.
-
-        Returns:
-            dict with centers as keys and ±1 as values
-        """
-        if centers is None:
-            centers = self._get_stereogenic_centers()
-        assert isinstance(centers, list)
-
-        results = {}
-        for center in centers:
-            neighbors = list(self.bonds[center])
-            neighbors.sort()
-            assert len(neighbors) >= 4, f"atom {center} has fewer than 4 neighbors ({neighbors})!"
-            results[center] = compute_chirality(*[self.get_vector(n, center) for n in neighbors])
-
-        return results
-
-    def exchange_identical_substituents(self, center, self_permutations=None):
-        """
-        Replace homotopic/enantiotopic/diastereotopic substituents about a single atom.
-
-        If a list of permuted ``Molecule`` objects is passed (as ``self_permutations``), then this code will apply this to each member and return a list.
-
-        Args:
-            center (integer): atomic number of atom to swap substituents around
-            self_permutations (list of Molecules): optional list of starting ``Molecule`` objects
-
-        Returns:
-            ``Molecule`` object (or list if ``self_permutations`` is not ``None``)
-        """
-        assert self.bonds.number_of_edges() > 0, "need a bond graph to perform this operation -- try calling self.assign_connectivity()!"
-        self._add_atomic_numbers_to_nodes()
-        neighbors = list(self.bonds[center])
-
-        returns = [copy.deepcopy(self)]
-        if self_permutations is not None:
-            returns = self_permutations
-
-
-        for i in range(len(neighbors)):
-            for j in range(i+1, len(neighbors)):
-                try:
-                    _, frag1 = self._get_bond_fragments(center, neighbors[i])
-                    _, frag2 = self._get_bond_fragments(center, neighbors[j])
-
-                    graph1 = self.bonds.subgraph(frag1)
-                    graph2 = self.bonds.subgraph(frag2)
-
-                    nm = nx.algorithms.isomorphism.categorical_node_match("atomic_number", 0)
-                    match = nx.algorithms.isomorphism.GraphMatcher(graph1, graph2, node_match=nm)
-                    if match.is_isomorphic():
-                        for mol in returns:
-                            new_mol = copy.deepcopy(mol)
-                            for k,v in match.mapping.items():
-                                new_mol = new_mol.swap_atom_numbers(k, v)
-                            if self_permutations is None:
-                                return new_mol
-
-                        returns.append(new_mol)
-
-                except ValueError as e:
-                    pass # probably indicates a cycle
-
-        if self_permutations is None:
-            raise ValueError("could not find substituents to switch")
-        else:
-            return returns
-
     def _add_atomic_numbers_to_nodes(self):
         """
         Add the atomic numbers to each node attribute, to allow for distinguishment of F and H during graph renumbering.
@@ -1512,84 +1456,6 @@ class Molecule:
                 return True
         return False
 
-    def flip_meso_rings(self, atoms):
-        """
-        Returns a list of permuted molecules with various ``meso`` rings renumbered.
-
-        Args:
-            atoms (list): atomic numbers of potential atoms to consider
-
-        Returns:
-            list of ``Molecule`` objects
-        """
-        #### get all rings in graph
-        returns = [copy.deepcopy(self)]
-        for center in atoms:
-            cycles = nx.cycle_basis(self.bonds, root=center)
-            for cycle in cycles:
-                #### get the correct ring
-                if center not in cycle:
-                    continue
-
-                #### reorder to put ``center`` first
-                while cycle[0] != center:
-                    # why yes, this /is/ a O(n) solution for reordering a list. why do you ask?
-                    cycle = cycle[1:] + cycle[0:1]
-                assert cycle[0] == center, "graph reorder failed"
-
-                #### create fragments
-                frag1 = [cycle.pop(1)]
-                frag2 = [cycle.pop(-1)]
-                while len(cycle) > 2:
-                    frag1.append(cycle.pop(1))
-                    frag2.append(cycle.pop(-1))
-
-                #### cut fragment bonds, depending on if we have even- or odd-numbered ring
-                new_returns = []
-                for mol in returns:
-                    mol = copy.deepcopy(mol)
-                    mol.remove_bond(frag1[0], cycle[0])
-                    mol.remove_bond(frag2[0], cycle[0])
-                    if len(cycle) == 1:
-                        mol.remove_bond(frag1[-1], frag2[-1])
-                    elif len(cycle) == 2:
-                        mol.remove_bond(frag1[-1], cycle[-1])
-                        mol.remove_bond(frag2[-1], cycle[-1])
-
-                    #### generate graphs
-                    graph1 = None
-                    graph2 = None
-                    fragments = nx.connected_components(mol.bonds)
-                    for fragment in fragments:
-                        if frag1[0] in fragment:
-                            graph1 = mol.bonds.subgraph(fragment)
-                        if frag2[0] in fragment:
-                            graph2 = mol.bonds.subgraph(fragment)
-
-                    assert isinstance(graph1, nx.Graph), "can't find graph 1"
-                    assert isinstance(graph2, nx.Graph), "can't find graph 1"
-
-                    #### do our two ring-halves match?? if so, we swap them
-                    nm = nx.algorithms.isomorphism.categorical_node_match("atomic_number", 0)
-                    match = nx.algorithms.isomorphism.GraphMatcher(graph1, graph2, node_match=nm)
-
-                    if match.is_isomorphic():
-                        for k,v in match.mapping.items():
-                            mol = mol.swap_atom_numbers(k, v)
-
-                        #### redo all the bonds we ablated
-                        if len(cycle) == 1:
-                            mol.add_bond(frag1[-1], frag2[-1], self.get_bond_order(frag1[-1], frag2[-1]))
-                        elif len(cycle) == 2:
-                            mol.add_bond(frag1[-1], cycle[-1], self.get_bond_order(frag1[-1], cycle[-1]))
-                            mol.add_bond(frag2[-1], cycle[-1], self.get_bond_order(frag2[-1], cycle[-1]))
-                        mol.add_bond(frag1[0], cycle[0], self.get_bond_order(frag1[0], cycle[0]))
-                        mol.add_bond(frag2[0], cycle[0], self.get_bond_order(frag2[0], cycle[0]))
-
-                        new_returns.append(mol)
-                returns = returns + new_returns
-        return returns
-
     def get_components(self):
         """
         Returns a list of all the connected components in a molecule.
@@ -1667,3 +1533,94 @@ class Molecule:
             self.geometry[f] += -1 * np.floor_divide(centroid, side_length) * side_length
 
         return self
+
+    @classmethod
+    def new_from_name(cls, name):
+        """
+        Create a new ``Molecule`` instance using ``rdkit``.
+        """
+        assert isinstance(name, str)
+        from urllib.request import urlopen
+
+        try:
+            url_name = re.sub(" ", "%20", name)
+            url = 'http://cactus.nci.nih.gov/chemical/structure/' + url_name + '/smiles'
+            smiles = urlopen(url).read().decode('utf8')
+            return cls.new_from_smiles(smiles)
+        except Exception as e:
+            raise ValueError(f"something went wrong auto-generating molecule {name}:\nurl: {url}\n{e}")
+
+    @classmethod
+    def new_from_smiles(cls, smiles):
+        """
+        Create a new ``Molecule`` instance using ``rdkit``.
+        """
+        assert isinstance(smiles, str)
+
+        try:
+            from rdkit.Chem import AllChem as Chem
+        except ImportError as e:
+            raise ImportError(f"``rdkit`` must be installed for this function to work!\n{e}")
+
+        try:
+            rdkm = Chem.MolFromSmiles(smiles)
+            rdkm = Chem.AddHs(rdkm)
+            Chem.EmbedMolecule(rdkm)
+            Chem.MMFFOptimizeMolecule(rdkm)
+
+            nums = []
+            for atom in rdkm.GetAtoms():
+                nums.append(atom.GetAtomicNum())
+            geom = rdkm.GetConformers()[0].GetPositions()
+
+            return cls(nums, geom)
+
+        except Exception as e:
+            raise ValueError(f"something went wrong auto-generating molecule {smiles}:\n{e}")
+
+    def fragment(self):
+        """
+        Returns list of ``cctk.Molecule`` objects based on the bond-connected components of ``self``.
+        """
+        fragments = list()
+        indices = self.get_components()
+        for idx in indices:
+            mol = cctk.Molecule(self.atomic_numbers[idx], self.geometry[idx]).assign_connectivity()
+            fragments.append(mol)
+        return fragments
+
+    def get_symmetric_atoms(self):
+        """
+        Returns lists of symmetric atoms, as defined in ``cctk.load_group``.
+
+        Useful for NMR spectroscopy, etc.
+        """
+        from cctk.load_groups import group_iterator
+
+        symmetric_sets = []
+        for group in group_iterator(symmetric_only=True):
+            # this gives us a list of dictionaries mapping from self.atomic_numbers to group numbers
+            matches = top.find_group(self, group)
+
+            for m in matches:
+                i = {v: k for k,v in m.items()}
+                for n in group.isomorphic:
+                    symmetric_sets.append([i[idx] for idx in n])
+
+        #### some groups overlap (e.g. methyl and t-butyl), so now we collapse the overlapping sets
+        for i, s1 in enumerate(symmetric_sets):
+            for j, s2 in enumerate(symmetric_sets[i+1:]):
+                if set(s1).intersection(set(s2)):
+                    symmetric_sets[i + j + 1] = list(set(s1).union(s2))
+                    symmetric_sets[i] = None # can't delete yet - messes up indexing
+
+        #### now we delete
+        symmetric_sets = list(filter(None, symmetric_sets))
+        return symmetric_sets
+
+    def atomic_symbols(self):
+        """
+        Return list of atomic symbols.
+        """
+        symbols = {z: get_symbol(z) for z in set(self.atomic_numbers)}
+        return [symbols[z] for z in self.atomic_numbers]
