@@ -509,7 +509,7 @@ def extract_success_and_time(lines):
         elapsed_time += fields[0] * 86400 + fields[1] * 3600 + fields[2] * 60 + fields[3]
     return len(success), elapsed_time
 
-def read_file_fast(file_text, filename, link1idx, max_len=1000, extended_opt_info=False):
+def read_file_fast(file_text, filename, link1idx, max_len=10000, extended_opt_info=False):
 
     #### "Make your bottleneck routines fast, everything else clear" - M. Scott Shell, UCSB
     #### Welcome to the fast part!
@@ -554,6 +554,7 @@ def read_file_fast(file_text, filename, link1idx, max_len=1000, extended_opt_inf
         ["Hirshfeld charges, spin densities, dipoles, and CM5 charges", " Hirshfeld charges", 1],
         ["Mulliken charges", "Sum of Mulliken charges", 1],
         ["Electronic spatial extent", "Quadrupole moment", 1], #10
+        ["normal coordinates", "Thermochemistry", 1],
     ]
 
     word_matches = [[] for _ in words]
@@ -709,11 +710,24 @@ def read_file_fast(file_text, filename, link1idx, max_len=1000, extended_opt_inf
         elif len(gibbs_vals) > 1:
             raise ValueError(f"unexpected # gibbs free energies found!\ngibbs free energies = {gibbs_vals}")
 
+
+        vibrational_modes = parse_modes(block_matches[11], num_atoms=molecules[-1].num_atoms(), hpmodes=re.search("hpmodes", route_card))
+        molecules[-1].vibrational_modes = vibrational_modes
+
         frequencies = []
         try:
             frequencies += extract_parameter(word_matches[14], 2)
-            frequencies += extract_parameter(word_matches[14], 3)
-            frequencies += extract_parameter(word_matches[14], 4)
+
+            # very small molecules might only have 1 or 2 freqs
+            try:
+                frequencies += extract_parameter(word_matches[14], 3)
+            except:
+                pass
+            try:
+                frequencies += extract_parameter(word_matches[14], 4)
+            except:
+                pass
+
             properties[-1]["frequencies"] = sorted(frequencies)
         except Exception as e:
             raise ValueError("error finding frequencies")
@@ -909,3 +923,93 @@ def parse_hirshfeld(hirshfeld_block):
 
     return cctk.OneIndexedArray(charges), cctk.OneIndexedArray(spins)
 
+def parse_modes(freq_block, num_atoms, hpmodes=False):
+    freqs = list()
+    masses = list()
+    force_ks = list()
+    displacements = list()
+
+    if len(freq_block) == 0:
+        return None
+
+    chunks = freq_block[0].split("Freq")
+
+    if hpmodes:
+        chunks = chunks[1:]
+
+    for chunk in chunks:
+        lines = chunk.split("\n")
+
+        if hpmodes:
+            num_cols = len(re.split(" +", lines[0])) - 2
+            current_displacements = [np.zeros(shape=(num_atoms, 3)) for x in range(num_cols)]
+
+            if len(freqs):
+                new_freqs = list(filter(None, re.split(" +", lines[0])))[2:]
+
+                if float(new_freqs[-1]) < float(freqs[-1]):
+                    break # want to skip the non-hpmodes section, so no looping allowed
+                else:
+                    freqs += new_freqs
+            else:
+                freqs += list(filter(None, re.split(" +", lines[0])))[2:]
+
+            masses += list(filter(None, re.split(" +", lines[1])))[3:]
+            force_ks += list(filter(None, re.split(" +", lines[2])))[3:]
+
+            for line in lines[6:]:
+                fields = re.split(" +", line)
+                fields = list(filter(None, fields))
+
+                if len(fields) < (num_cols + 3):
+                    continue
+
+                if fields[0] == "Harmonic":
+                    break
+
+                for col_idx, val in enumerate(fields[3:]):
+                    current_displacements[col_idx][int(fields[1])-1][int(fields[0])-1] = val
+
+            for d in current_displacements:
+                displacements.append(d.view(cctk.OneIndexedArray))
+
+        else:
+            current_displacements = [list() for x in re.split(" +", lines[0])[2:]]
+
+            freqs += re.split(" +", lines[0])[2:]
+            masses += re.split(" +", lines[1])[4:]
+            force_ks += re.split(" +", lines[2])[4:]
+
+            for line in lines[5:]:
+                fields = re.split(" +", line)
+                fields = list(filter(None, fields))
+
+                if len(fields) < 4:
+                    break
+
+                current_displacements[0].append([float(x) for x in fields[2:5]])
+
+                if len(current_displacements) > 1:
+                    current_displacements[1].append([float(x) for x in fields[5:8]])
+
+                if len(current_displacements) > 2:
+                    current_displacements[2].append([float(x) for x in fields[8:11]])
+
+
+            for d in current_displacements:
+                displacements.append(cctk.OneIndexedArray(d))
+
+    freqs = [float(x) for x in freqs]
+    masses = [float(x) for x in masses]
+    force_ks = [float(x) for x in force_ks]
+
+    assert len(freqs) == len(masses)
+    assert len(freqs) == len(force_ks)
+    assert len(freqs) == len(displacements)
+
+    modes = list()
+    for f, m, k, d in zip(freqs, masses, force_ks, displacements):
+        k *= 143.9326 # mdyne Å**-1 to kcal/mol Å**-2
+        modes.append(cctk.VibrationalMode(frequency=f, reduced_mass=m, force_constant=k, displacements=d))
+
+    return modes
