@@ -2,6 +2,7 @@ import unittest, sys, os, io, copy, math
 import numpy as np
 import networkx as nx
 import cctk
+import cctk.topology as top
 
 class TestMolecule(unittest.TestCase):
     def load_molecule(self, path="test/static/test_peptide.xyz"):
@@ -151,8 +152,8 @@ class TestMolecule(unittest.TestCase):
         self.assertEqual(m3.multiplicity, 1)
 
     def test_volume(self):
-        mol = self.load_molecule()
-        self.assertEqual(mol.volume(), 80.42662516224424)
+        mol = cctk.Molecule.new_from_name("acetone")
+        self.assertTrue(abs(mol.volume() - 62) < 1)
 
     def test_renumber(self):
         mol = self.load_molecule()
@@ -167,9 +168,9 @@ class TestMolecule(unittest.TestCase):
 
     def test_chirality(self):
         mol = self.load_molecule().assign_connectivity()
-        self.assertListEqual(mol._get_stereogenic_centers(), [3, 5, 11, 13, 18, 28])
+        self.assertListEqual(top.get_stereogenic_centers(mol), [3, 5, 11, 13, 18, 28])
 
-        mol.get_chirality_report()
+        top.get_chirality_report(mol)
 
     def test_periodic_boundary_conditions(self):
         m1 = cctk.Molecule([9, 9], [[0, 5, 4.5], [0, 0, 0.5]], charge=0, multiplicity=1)
@@ -187,5 +188,23 @@ class TestMolecule(unittest.TestCase):
         m3 = m2.limit_solvent_shell(num_solvents=10)
         self.assertEqual(m3.num_atoms(), 83)
 
+    def test_rdkit(self):
+        mol = cctk.Molecule.new_from_name("acetone")
+        self.assertEqual(len(mol.atomic_numbers), 10)
+
+        mol = cctk.Molecule.new_from_name("acetic acid")
+        self.assertEqual(len(mol.atomic_numbers), 8)
+
+    def test_fragment(self):
+        m = cctk.GaussianFile.read_file("test/static/periodic.gjf").get_molecule()
+        m = m.assign_connectivity(periodic_boundary_conditions=np.array([20, 20, 20]))
+        f = m.fragment()
+
+        ibuprofen = cctk.Molecule.new_from_name("ibuprofen").assign_connectivity()
+        chloroform = cctk.Molecule.new_from_name("chloroform").assign_connectivity()
+
+        self.assertTrue(top.are_isomorphic(f[0], ibuprofen))
+        self.assertTrue(top.are_isomorphic(f[1], chloroform))
+
 if __name__ == '__main__':
     unittest.main()
